/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2018-2022 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::laminarModels::generalisedNewtonianViscosityModels::shiftedPowerLaw

Description
    Standard power-law generalised Newtonian viscosity model with a shift factor

    The power-law model provides a function for viscosity, limited by
    minimum and maximum values \f$\nu_{\min}\f$ and \f$\nu_{\max}\f$
    respectively.

    Kinematic viscosity [m^2/s]

    \f[
        \nu = k\gamma^{(n - 1)/shifFactor}, \nu_{\min} \le \nu \le \nu_{\max}
    \f]

    Example specification:
    \verbatim
         viscosityModel shiftedPowerLaw;

         nuMax      1e-3;
         nuMin      1e-5;
         k          1;
         n          0.5;
         shiftFactor  1;
    \endverbatim

SourceFiles
    shiftedPowerLaw.C

\*---------------------------------------------------------------------------*/

#ifndef shiftedPowerLaw_H
#define shiftedPowerLaw_H

#include "strainRateViscosityModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace laminarModels
{
namespace generalisedNewtonianViscosityModels
{

/*---------------------------------------------------------------------------*\
                          Class shiftedPowerLaw Declaration
\*---------------------------------------------------------------------------*/

class shiftedPowerLaw
:
    public strainRateViscosityModel
{
    // Private Data

        dimensionedScalar k_;
        dimensionedScalar n_;
        dimensionedScalar nuMin_;
        dimensionedScalar nuMax_;
        dimensionedScalar shiftFactor_;


public:

    //- Runtime type information
    TypeName("shiftedPowerLaw");


    // Constructors

        //- Construct from components
        shiftedPowerLaw
        (
            const dictionary& viscosityProperties,
            const Foam::viscosity& viscosity,
            const volVectorField& U
        );


    //- Destructor
    virtual ~shiftedPowerLaw()
    {}


    // Member Functions

        //- Read transportProperties dictionary
        virtual bool read(const dictionary& viscosityProperties);

        //- Return the laminar viscosity
        virtual tmp<volScalarField> nu
        (
            const volScalarField& nu0,
            const volScalarField& strainRate
        ) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace generalisedNewtonianViscosityModels
} // End namespace laminarModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
